from jyhelper import timeHelper
from jyhelper.sysHelper import sysHelper
import hashlib
from urllib.parse import quote, unquote
import concurrent.futures
import base64
import random
import re


class common:

    # 打印日志
    @staticmethod
    def debug(*args, path=None):
        path = './debug.log' if path is None else path
        with open(path, 'a', encoding='utf-8') as f:
            f.write("\n---------------" + timeHelper.getDate() + "---------------\n")
            for arg in args:
                f.write(str(arg) + "\n")

    # 带时间输出
    @staticmethod
    def print(*args):
        print(timeHelper.getDate(), '--->', *args)

    def print_in_line(*args, show_time=True, show_ok=None):
        """
        在一行输出
        :param args:
        :param show_time:是否显示时间
        :param show_ok:是否显示成功表示
        :return:
        """
        param = list(args)
        if show_time:
            param = [timeHelper.getDate(), '--->'] + param
        if show_ok:
            if isinstance(show_ok, str):
                param.append(show_ok)
            else:
                param.append(sysHelper.green('[OK]'))

        param[0] = "\r" + param[0]
        print(*param, end='', flush=True)

    @staticmethod
    def print_table(data, table_name=None, data_has_title=True, show_num=True, sort_col_index=None,sort_lambda=None,sort_reverse=False):
        """
        打印输出整齐的表格
        :param data:待打印的二维数组
        :param table_name:表格名字
        :param data_has_title:数据是否包含标题
        :param show_num:是否显示数据总量在标提上
        :param sort_col_index:排序列 0 1 2...
        :param sort_lambda:排序的函数，默认按字符串顺序排列 x表示那一列那个表格的值 sort_lambda=lambda x:len(x)
        :param sort_reverse:排列顺序 默认正序排列
        :return:
        """
        if not bool(data):
            data = [['No Data']]

        def delStrColor(string):
            string = str(string)
            string = string.replace('\033[91m', '')
            string = string.replace('\033[0m', '')
            string = string.replace('\033[92m', '')
            return string

        def getRowWidth(string):
            """
            此函数使用正则表达式计算字符串 s 的长度，其中汉字及中文标点算 1.5 个长度，英文及英文标点算 1 个长度
            """
            string = str(string)
            # 去掉颜色的标识
            string = delStrColor(string)
            # 查找所有汉字和中文标点
            chinese_chars = re.findall(r'[\u4e00-\u9fff\u3000-\u303f]', string)
            # 总长度 = 汉字和中文标点的数量 * 2 + 其余字符的数量
            length = len(chinese_chars) * 1.5 + (len(string) - len(chinese_chars))
            return round(length)

        # 对数据排序
        if sort_col_index is not None and data != [['No Data']]:
            if sort_lambda is None:
                def sort_lambda(x):
                    return x
            if data_has_title:
                data = data[:1] + sorted(data[1:], key=lambda x: sort_lambda(delStrColor(x[sort_col_index])),reverse=sort_reverse)
            else:
                data = sorted(data, key=lambda x: sort_lambda(delStrColor(x[sort_col_index])),reverse=sort_reverse)

        # 得到每一列的宽度
        widths = []
        for i in range(len(data[0])):
            widths.append(0)
        for row in data:
            for index, item in enumerate(row):
                thisWidth = getRowWidth(item)
                if thisWidth > widths[index]:
                    widths[index] = thisWidth
        if table_name is None:
            table_name = ''
        if show_num:
            table_name += " - Num: %s" % (data.__len__() - 1 if data_has_title else data.__len__())
        if bool(table_name):
            titleWidth = sum(widths)
            titleWidth += (len(data[0]) - 1) * 3 + 4
            print('+' + (titleWidth - 2) * '-' + '+')
            titleEmpty = round((titleWidth - 2 - getRowWidth(table_name)) / 2)
            print('|' + titleEmpty * ' ' + table_name + (
                    titleWidth - 2 - getRowWidth(table_name) - titleEmpty) * ' ' + '|')
        # 打印
        i = 0
        dataLen = len(data)
        for row in data:
            printRow = ""
            fenGe = ""
            for index, item in enumerate(row):
                item = str(item)
                printOneItem = (f"\033[1m{item}\033[0m" if i == 0 and data_has_title else item) + (widths[index] - getRowWidth(item)) * ' '
                printOneItem = '| ' + printOneItem + ' '
                printRow += printOneItem
                if i == 0 or i == dataLen - 1:
                    fenGeOneItem = widths[index] * '-'
                    fenGeOneItem = '+ ' + fenGeOneItem + ' '
                    fenGe += fenGeOneItem
            printRow += "|"
            if i == 0:
                fenGe += "+"
                print(fenGe)
            print(printRow)
            if i == dataLen - 1:
                if dataLen - 1 != 0:
                    fenGe += "+"
                print(fenGe)
            i += 1

    # 将列表分割 每一份n的长度
    @staticmethod
    def explodeList(data, n):
        if isinstance(data, list):
            return [data[i:i + n] for i in range(0, len(data), n)]
        else:
            return []

    @staticmethod
    def re_split(string, pattern=None):
        """默认用空格分割字符串，多个空格会合并成一个处理"""
        pattern = r'\s+' if pattern is None else pattern
        return re.split(pattern, string)

    # 把英文的引号转换程中文的引号
    @staticmethod
    def replaceYinHao(strings):
        return strings.replace('"', '“').replace("'", "‘")

    # 把值转为int
    @staticmethod
    def transInt(val, default=0):
        try:
            val = int(val)
        except ValueError:
            val = default
        return val

    @staticmethod
    def transFloat(val, default=0):
        try:
            val = float(val)
        except ValueError:
            val = default
        return val

    # 转化为保留两位小数的格式
    @staticmethod
    def switch2(data1, data2=None, returnDefault=0.00, precision=2):
        if data1 == 0 or data2 == 0:
            return returnDefault
        if data2 is not None:
            data = data1 / data2
        else:
            data = data1
        return round(data, precision)

    # 转换为百分比
    @staticmethod
    def switchRate(data1, data2=None, returnDefault='0.00%', precision=2):
        if data1 == 0 or data2 == 0:
            return returnDefault
        if data2 is not None:
            data = data1 / data2 * 100
        else:
            data = data1 * 100
        return ('{:.%sf}' % precision).format(data) + '%'

    # 从list中删除数据
    @staticmethod
    def delListValue(needList, delValues):
        if not isinstance(delValues, list):
            delValues = [delValues]
        for delValue in delValues:
            needList = [x for x in needList if x != delValue]
        return needList

    # 查找二维数组的某一行 demo [{'iType': 1, 'num': 3972}, {'iType': 2, 'num': 6315}, {'iType': 3, 'num': 6250}]
    @staticmethod
    def search2ListRow(data, keyField, value):
        for row in data:
            if row[keyField] == value:
                return row
        return {}

    # 把二维数组根据某一个key转为字典 demo [{'iType': 1, 'num': 3972}, {'iType': 2, 'num': 6315}, {'iType': 3, 'num': 6250}]
    @staticmethod
    def trans2ListDict(data, keyField):
        retDict = {}
        for row in data:
            retDict[row[keyField]] = row
        return retDict

    # 排序字典的key
    @staticmethod
    def sortDictByKey(myDict, reverse=False):
        return dict(sorted(myDict.items(), key=lambda x: x[0], reverse=reverse))

    # 排序字典的value
    @staticmethod
    def sortDictByValue(myDict, reverse=False):
        return dict(sorted(myDict.items(), key=lambda x: x[1], reverse=reverse))

    """
    根据条件，删除字典中的数据
    myDict = {'key1': 1, 'key2': 2, 'key3': 3}
    common.delDictItem(myDict, "key == 'key1'")
    common.delDictItem(myDict, "value == 1")
    common.delDictItem(myDict, "value > 1")
    """

    @staticmethod
    def delDictItem(myDict=None, delWhere=None):
        if not isinstance(myDict, dict):
            return {}
        if delWhere is None:
            return myDict
        # 使用字典推导式创建新字典，排除要删除的键
        myDict = {key: value for key, value in myDict.items() if not eval(delWhere)}
        return myDict

    @staticmethod
    def getMD5(string):
        string = string if isinstance(string, str) else str(string)
        md5 = hashlib.md5()
        md5.update(string.encode('utf-8'))
        return md5.hexdigest()

    @staticmethod
    def urlEncode(string):
        string = string if isinstance(string, str) else str(string)
        return quote(string, 'utf-8')

    @staticmethod
    def urlDecode(string):
        string = string if isinstance(string, str) else str(string)
        return unquote(string, 'utf-8')

    # 多线程运行任务
    """
    tasks = [[funcName, p1,p2],[funcName, p1,p2],[funcName, p1,p2],[funcName, p1,p2]]
    返回的exception里面是None就表示正常
    返回的result里面有None，大概率就是出现异常了 None not in result 表示全部正常
    """

    @staticmethod
    def multipleThreadRunTasks(max_workers, tasks, get_result=False, get_exceptions=False, print_info=True):
        # 创建一个拥有 max_workers 个线程的线程池
        with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
            futures = []
            for oneTask in tasks:
                #  executor.submit(funcName, p1, p2,..)
                future = executor.submit(*oneTask)
                futures.append(future)
            # 监控线程池中剩余的任务数量
            all_futures = futures.__len__()
            while True:
                timeHelper.sleep(1)
                unfinished_count = sum(not future.done() for future in futures)
                if print_info:
                    common.print(f"完成进度: {all_futures - unfinished_count}/{all_futures}")
                if unfinished_count == 0:
                    break
            results = []
            exceptions = []
            if get_result:
                for future in futures:
                    if future.exception() is None:
                        results.append(future.result())
                    else:
                        results.append(None)
            if get_exceptions:
                for future in futures:
                    exceptions.append(future.exception())
            if get_result and not get_exceptions:
                return results
            if not get_result and get_exceptions:
                return exceptions
            if get_result and get_exceptions:
                return results, exceptions
            return None

    # 多进程运行任务
    @staticmethod
    def multipleProcessRunTasks(max_workers, tasks, get_result=False, get_exceptions=False, print_info=True):
        # 创建一个拥有 max_workers 个线程的线程池
        with concurrent.futures.ProcessPoolExecutor(max_workers=max_workers) as executor:
            futures = []
            for oneTask in tasks:
                #  executor.submit(funcName, p1, p2,..)
                future = executor.submit(*oneTask)
                futures.append(future)
            # 监控线程池中剩余的任务数量
            all_futures = futures.__len__()
            while True:
                timeHelper.sleep(1)
                unfinished_count = sum(not future.done() for future in futures)
                if print_info:
                    common.print(f"完成进度: {all_futures - unfinished_count}/{all_futures}")
                if unfinished_count == 0:
                    break
            results = []
            exceptions = []
            if get_result:
                for future in futures:
                    if future.exception() is None:
                        results.append(future.result())
                    else:
                        results.append(None)
            if get_exceptions:
                for future in futures:
                    exceptions.append(future.exception())
            if get_result and not get_exceptions:
                return results
            if not get_result and get_exceptions:
                return exceptions
            if get_result and get_exceptions:
                return results, exceptions
            return None

    # base64编码
    @staticmethod
    def base64Encode(string):
        string = str(string) if not isinstance(string, str) else string
        return base64.b64encode(string.encode('utf-8')).decode('utf-8')

    # Base64 编码在最后可能需要添加填充字符 '=' 以使编码后的字符串长度是 4 的倍数。如果输入的字符串没有正确的填充，就会导致解码错误。
    @staticmethod
    def base64Decode(string):
        string = str(string)
        stringLen = string.__len__()
        yuShu = stringLen % 4
        addString = ''
        if yuShu > 0:
            addString = (4 - yuShu) * '='
        return base64.b64decode(string + addString).decode('utf-8')

    # 得到一个随机整数，包含start和end
    @staticmethod
    def getRandomInt(startInt, endInt):
        return random.randint(startInt, endInt)


if __name__ == '__main__':
    # print(common.re_split('1    2  3 4'))

    common.print_table(data=[
        ["Nameaaa", "Age", "ZCity"],
        [sysHelper.red("Alice"), "25", "New York中文"],
        ["Boba", "30", "San Francisco"],
        [sysHelper.red("Chardlie"), "22", "Los Angeles"],
        ["Charlie", "22", sysHelper.green("Los Angeles")],
        ["Charlieaa", "22", "Los Angeles"],
        ["Charlieaa", "22", "Los Angeles"],
    ], table_name='list of names', data_has_title=True, sort_col_index=0,sort_lambda=lambda x:len(x),sort_reverse=True)

    # print(common.trans2ListDict([{'iType': 1, 'num': 3972}, {'iType': 2, 'num': 6315}, {'iType': 3, 'num': 6250}],'iType'))

    """
    def test(p1):
        return p1

    tasks = []
    for i in range(100):
        tasks.append([common.urlEncode, i])
    common.print('start')
    result = common.multipleThreadRunTasks(50,tasks=tasks,get_result=True)
    print(result)
    common.print('end')
    exit()
    """

    # test = ['a','b','a','c','d',None]
    # print(common.delListValue(test,['f',None,'a']))
    # print(test)
    # print(common.delDictItem({'c_actual收入': 900.0, 'a日期': '2023-11-09', 'b新增': 62, 'd留存1': 62, 'e收入1': 0, 'f付费人数1': 0, 'd留存2': 0, 'e收入2': 0, 'f付费人数2': 0, 'd留存3': 0, 'e收入3': 0, 'f付费人数3': 0},"key=='c_actual收入' and value==900"))
    # print(common.switchRate(0,0,precision=4,returnDefault='0.0000%'))
    # print(common.urlDecode('%E4%BD%A0%E5%A5%BD'))



#! /usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time : 2025/01/13 18:35 
# @Author : JY
"""
系统相关的操作
"""

import concurrent.futures
import datetime
import json
import locale
import subprocess
import sys
import time


class sysHelper:

    @staticmethod
    def run_command(command, printInfo=True, returnStr=False, returnJson=False, encoding_utf8=False, encoding_gbk=False,
                    timeout=None):
        """
        :param command: 执行的命令
        :param printInfo: 是否打印info信息
        :param returnStr: 返回字符串 默认返回list
        :param returnJson: 返回json解析后的python可直接操作对象
        :param encoding_utf8:
        :param encoding_gbk:
        :param timeout: 超时时间 默认None不超时，可以设定一个秒数
        :return: 默认list 可选字符串和json解析后的对象
        """
        res_lines = []
        encoding = locale.getpreferredencoding(False)
        if encoding_utf8:
            encoding = 'utf-8'
        if encoding_gbk:
            encoding = 'gbk'
        if printInfo:
            print('info', 'run:', command)
        start_time = time.time()
        # 3.7以下的版本 text=True 报错，修改为universal_newlines=True
        process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True,
                                   universal_newlines=True,
                                   encoding=encoding)  # stderr=subprocess.PIPE,可以捕获错误，不设置就是直接输出
        printRes = False
        while True:
            if timeout is not None and time.time() - start_time > timeout:
                process.kill()
                print(sysHelper.red('Error'), sysHelper.red(f"Max allow {timeout} seconds, Time out!"))
                return None

            output = process.stdout.readline()
            if output == '' and process.poll() is not None:
                break
            if output:
                outStr = output.strip()
                res_lines.append(outStr)
                if printInfo:
                    if not printRes:
                        print('info', 'res:')
                        printRes = True
                    print(outStr)
                sys.stdout.flush()

        if returnStr:
            res_lines = '\n'.join(res_lines)
        if returnJson:
            res_lines = ''.join(res_lines)
            res_lines = json.loads(res_lines)
        exit_code = process.poll()
        if exit_code != 0:
            print(sysHelper.red('Error: '))
            for line in process.stderr:
                print(sysHelper.red(line.strip()))
            if isinstance(res_lines, list):
                for row in res_lines:
                    print(sysHelper.red(row))
            else:
                print(sysHelper.red(str(res_lines)))
            return None
        return res_lines

    @staticmethod
    def red(msg):
        """print以后会显示亮红色字体"""
        return f"\033[91m{msg}\033[0m"

    @staticmethod
    def green(msg):
        """print以后会显示亮绿色字体"""
        return f"\033[92m{msg}\033[0m"

    @staticmethod
    def display_all_colors():
        """
        显示全部可用的颜色
        :return:
        """
        j = 0
        for i in range(108):
            if i in [2, 3, 5, 6, 8, 38, 39, 48, 49, 50, 98] or 10 <= i <= 20 or 22 <= i <= 29 or 52 <= i <= 89:
                continue
            msg = f"\033[{i}m" + r"\033[%sm内容\033[0m" % i + "\033[0m"
            if i < 10:
                msg += " "
            if i < 100:
                msg += " "
            print(msg, '', '', end='')
            j += 1
            if j % 8 == 0:
                print()

    @staticmethod
    def logError(msg1='', msg2='', msg3=''):
        """1个参数默认就是红色，2个或者3个参数msg2是红色"""
        if msg2 == '' and msg3 == '':
            print(sysHelper.red(msg1))
        elif msg2 != '' and msg3 == '':
            print(msg1, sysHelper.red(msg2))
        elif msg2 != '' and msg3 != '':
            print(msg1, sysHelper.red(msg2), msg3)

    @staticmethod
    def logInfo(msg1='', msg2='', msg3=''):
        if msg2 == '' and msg3 == '':
            print(sysHelper.green(msg1))
        elif msg2 != '' and msg3 == '':
            print(msg1, sysHelper.green(msg2))
        elif msg2 != '' and msg3 != '':
            print(msg1, sysHelper.green(msg2), msg3)

    @staticmethod
    def input(prompt, not_empty=False, number_only=False, default_value=None, choice=None):
        """
        输入控制
        :param prompt: 显示给用户的提示
        :param not_empty: 不能为空
        :param number_only: 只能输入正整数
        :param default_value: 默认值
        :param choice: 只能从这里面选 可以是list 也可以是str str会默认循环
        :return: 输入
        """
        while True:
            if default_value is not None:
                if prompt.endswith(':'):
                    prompt = prompt[0:-1]
                prompt += "(Default=%s):" % default_value
            user_input = input(prompt).strip()
            if default_value is not None and not user_input:
                return default_value
            if not_empty and not user_input:
                print(sysHelper.red("输入无效,内容不能为空，请重新输入!"))
                continue
            if number_only and not user_input.isdigit():
                print(sysHelper.red("输入无效,内容只能是数字，请重新输入!"))
                continue
            if choice is not None:
                choice = [str(one_choice) for one_choice in choice]
                if user_input not in choice:
                    print(sysHelper.red(f"输入无效，请从 {choice} 中选择！"))
                    continue
            return user_input

    @staticmethod
    def run_tasks(task_func, task_params, max_workers=10, print_info=True, thread_mode=True, process_mode=False,
                  raise_exception=True):
        """
            任务类型	     推荐方案	         核心原因
            IO 密集型	 多线程           线程切换开销小，IO 等待时释放 GIL     爬虫（等待网页响应）、API 调用（等待返回结果）、日志写入（磁盘 IO 等待）
            CPU 密集型	 多进程	         突破 GIL 限制，利用多核并行计算       大规模数据排序、矩阵运算、视频帧处理
        :param task_func: 方法对象
        :param task_params: 参数数组 例: [['a','b'],['a','c'],['b','d']]
        :param max_workers: 线程/进程 数量
        :param print_info: 打印执行信息
        :param thread_mode: 线程模式(默认)
        :param process_mode:进程模式
        :param raise_exception: 执行遇到异常是否抛出
        :return: success, results, exceptions
        """
        if process_mode:
            thread_mode = False
        process_or_thread_pool = None  # 线程或进程池
        if thread_mode:
            process_or_thread_pool = concurrent.futures.ThreadPoolExecutor(max_workers=max_workers)
        if process_mode:
            process_or_thread_pool = concurrent.futures.ProcessPoolExecutor(max_workers=max_workers)
        if print_info:
            print(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'), '--->',"任务开始", task_func.__name__)
            print(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'), '--->',"任务数量", task_params.__len__())
        with process_or_thread_pool as executor:
            futures = []
            for one_param in task_params:
                #  executor.submit(funcName, p1, p2,..)
                future = executor.submit(task_func, *one_param)
                futures.append(future)

            # 监控线程池中剩余的任务数量
            all_futures = futures.__len__()
            while True:
                time.sleep(1)
                unfinished_count = sum(not future.done() for future in futures)
                if print_info:
                    print(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'), '--->',f"完成进度: {all_futures - unfinished_count}/{all_futures}")
                if unfinished_count == 0:
                    break
            results = []
            exceptions = []
            index = 0
            for future in futures:
                if future.exception() is None:
                    results.append(future.result())
                    exceptions.append(None)
                else:
                    results.append(None)
                    exceptions.append("param_index:%s %s -> %s -> %s" % (
                        index, task_func.__name__ + '(' + str(task_params[index])[1:-1] + ')',
                        type(future.exception()).__name__, str(future.exception())))
                index += 1
            success = True if sum(0 if one is None else 1 for one in exceptions) == 0 else False
            if raise_exception and not success:
                exception_str = "\n"
                for one in exceptions:
                    if one is not None:
                        exception_str += f"{one}\n"
                raise Exception(exception_str)
            if print_info:
                all_num = task_params.__len__()
                except_num = sum(0 if one is None else 1 for one in exceptions)
                success_num = all_num - except_num
                print(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'), '--->','执行完成', f"总数: {all_num}", f"成功: {success_num}", f"失败: {except_num}")
            return success, results, exceptions


if __name__ == '__main__':
    pass
    # cmd = 'ping www.abaidu.com'
    # res = sysHelper.run_command(cmd,returnStr=True)
    # print('--------返回的结果---------')
    # print(res)
    # res = sysHelper.input('清输入：',not_empty=False,number_only=False,choice='12345')
    # res = sysHelper.input('清输入',not_empty=False,number_only=False,choice='yn',default_value='y')
    # print(res)

    """
    params = [
        [1, 1],
        [1, 5],
        [1, 0],
        [1, 10],
    ]
    res = sysHelper.run_tasks(task_func=lambda a,b:a/b, task_params=params, thread_mode=True, raise_exception=False)
    print('-----------')
    print(res)
    """
